//------------------------------------------------------------------
#property copyright "www.forex-station.com"
#property link      "www.forex-station.com"
//------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers    2

#property indicator_label1     "Stochastic"
#property indicator_type1      DRAW_LINE
#property indicator_color1	    clrBlue
#property indicator_width1     2

#property indicator_label2     "Stochastic signal"
#property indicator_type2      DRAW_LINE
#property indicator_color2	    clrGray
#property indicator_style2     STYLE_DOT

#property indicator_level1     75
#property indicator_level2	    25
#property indicator_maximum    100
#property indicator_minimum    0
#property indicator_levelstyle STYLE_DOT
#property strict

//
//
//
//
//

extern ENUM_TIMEFRAMES TimeFrame       = PERIOD_CURRENT;                // Time frame
input int              StoKPeriod      = 14;                            // Stochastic K period
input int              StoDPeriod      = 3;                             // Stochastic D period
input int              StoSlowing      = 3;                             // Stochastic slowing
input ENUM_STO_PRICE   PriceField      = 0;                             // Stochastic price
input ENUM_MA_METHOD   StochMaMode     = MODE_SMA;                      // Stochastic ma mode  
input ENUM_MA_METHOD   SignalMaMode    = MODE_SMA;                      // Stochastic signal ma mode
input bool             alertsOn        = true;                          // Alerts on true/false?
input bool             alertsOnCurrent = false;                         // Alerts on open bar true/false?
input bool             alertsMessage   = true;                          // Alerts popup message true/false?
input bool             alertsSound     = false;                         // Alerts should play a sound?
input bool             alertsEmail     = false;                         // Alerts email true/false?
input bool             alertsPushNotif = false;                         // Alerts notification true/false?
input bool             Interpolate     = true;                          // Interpolate in mtf mode?

double sto[],sig[],trend[],count[];
string indicatorFileName;
#define _mtfCall(_buff,_ind) iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,StoKPeriod,StoDPeriod,StoSlowing,PriceField,StochMaMode,SignalMaMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsPushNotif,_buff,_ind)

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
// 
//
//
//
//

int OnInit()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,sto,  INDICATOR_DATA); 
   SetIndexBuffer(1,sig,  INDICATOR_DATA); 
   SetIndexBuffer(2,trend,INDICATOR_CALCULATIONS);
   SetIndexBuffer(3,count,INDICATOR_CALCULATIONS);
   
   indicatorFileName = WindowExpertName();
   TimeFrame         = fmax(TimeFrame,_Period);
  
   IndicatorShortName(timeFrameToString(TimeFrame)+" Stochastic ("+(string)StoKPeriod+","+(string)StoDPeriod+","+(string)StoSlowing+")");
return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason){  }

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

int OnCalculate(const int rates_total,const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tickVolume[],
                const long &volume[],
                const int &spread[])

{
   int i,counted_bars=prev_calculated;
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = fmin(rates_total-counted_bars,rates_total-1); count[0]=limit;
            if (TimeFrame!=_Period)
            {
               limit = (int)fmax(limit,fmin(rates_total-1,_mtfCall(3,0)*TimeFrame/_Period));
               for (i=limit; i>=0; i--)
               {
                  int y = iBarShift(NULL,TimeFrame,Time[i]);
                     sto[i] = _mtfCall(0,y);
                     sig[i] = _mtfCall(1,y);
                 
                     //
                     //
                     //
                     //
                     //
                     
                     if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;
                        #define _interpolate(buff) buff[i+k] = buff[i]+(buff[i+n]-buff[i])*k/n
                        int n,k; datetime dtime = iTime(NULL,TimeFrame,y);
                           for(n = 1; (i+n)<rates_total && Time[i+n] >= dtime; n++) continue;	
                           for(k = 1; k<n && (i+n)<rates_total && (i+k)<rates_total; k++) 
                           {
                              _interpolate(sto);
                              _interpolate(sig);
                           }                                           
              }      
   return(rates_total);
   }
      
   //
   //
   //
   //
   //

   for (i=limit; i>=0; i--)
   {
       sto[i] = iStochastic(NULL,0,StoKPeriod,StoDPeriod,StoSlowing,StochMaMode, PriceField,MODE_MAIN  ,i);
       sig[i] = iStochastic(NULL,0,StoKPeriod,StoDPeriod,StoSlowing,SignalMaMode,PriceField,MODE_SIGNAL,i);
       trend[i] = (i<rates_total-1) ? (sto[i]>sig[i]) ? 1 : (sto[i]<sig[i]) ? -1 : trend[i+1] : 0; 
          
   }
   
   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (trend[whichBar] != trend[whichBar+1])
      if (trend[whichBar] == 1)
            doAlert("crossing signal up");
      else  doAlert("crossing signal down");       
   }         
return(rates_total);
}

//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message = timeFrameToString(_Period)+" "+_Symbol+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" Stochastic "+doWhat;
             if (alertsMessage)   Alert(message);
             if (alertsPushNotif) SendNotification(message);
             if (alertsEmail)     SendMail(_Symbol+" Stochastic ",message);
             if (alertsSound)     PlaySound("alert2.wav");
      }
}


